# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .bill import Bill as Bill
from .check import Check as Check
from .class_ import Class as Class
from .vendor import Vendor as Vendor
from .account import Account as Account
from .invoice import Invoice as Invoice
from .customer import Customer as Customer
from .estimate import Estimate as Estimate
from .transfer import Transfer as Transfer
from .sales_order import SalesOrder as SalesOrder
from .service_item import ServiceItem as ServiceItem
from .journal_entry import JournalEntry as JournalEntry
from .sales_receipt import SalesReceipt as SalesReceipt
from .standard_term import StandardTerm as StandardTerm
from .inventory_item import InventoryItem as InventoryItem
from .inventory_site import InventorySite as InventorySite
from .purchase_order import PurchaseOrder as PurchaseOrder
from .sales_tax_code import SalesTaxCode as SalesTaxCode
from .sales_tax_item import SalesTaxItem as SalesTaxItem
from .receive_payment import ReceivePayment as ReceivePayment
from .bill_list_params import BillListParams as BillListParams
from .date_driven_term import DateDrivenTerm as DateDrivenTerm
from .check_list_params import CheckListParams as CheckListParams
from .class_list_params import ClassListParams as ClassListParams
from .bill_check_payment import BillCheckPayment as BillCheckPayment
from .bill_create_params import BillCreateParams as BillCreateParams
from .bill_update_params import BillUpdateParams as BillUpdateParams
from .credit_card_charge import CreditCardCharge as CreditCardCharge
from .credit_card_credit import CreditCardCredit as CreditCardCredit
from .non_inventory_item import NonInventoryItem as NonInventoryItem
from .vendor_list_params import VendorListParams as VendorListParams
from .account_list_params import AccountListParams as AccountListParams
from .check_create_params import CheckCreateParams as CheckCreateParams
from .check_update_params import CheckUpdateParams as CheckUpdateParams
from .class_create_params import ClassCreateParams as ClassCreateParams
from .class_list_response import ClassListResponse as ClassListResponse
from .class_update_params import ClassUpdateParams as ClassUpdateParams
from .invoice_list_params import InvoiceListParams as InvoiceListParams
from .customer_list_params import CustomerListParams as CustomerListParams
from .estimate_list_params import EstimateListParams as EstimateListParams
from .transfer_list_params import TransferListParams as TransferListParams
from .vendor_create_params import VendorCreateParams as VendorCreateParams
from .vendor_update_params import VendorUpdateParams as VendorUpdateParams
from .account_create_params import AccountCreateParams as AccountCreateParams
from .account_list_response import AccountListResponse as AccountListResponse
from .account_update_params import AccountUpdateParams as AccountUpdateParams
from .invoice_create_params import InvoiceCreateParams as InvoiceCreateParams
from .invoice_update_params import InvoiceUpdateParams as InvoiceUpdateParams
from .customer_create_params import CustomerCreateParams as CustomerCreateParams
from .customer_update_params import CustomerUpdateParams as CustomerUpdateParams
from .estimate_create_params import EstimateCreateParams as EstimateCreateParams
from .estimate_update_params import EstimateUpdateParams as EstimateUpdateParams
from .transfer_create_params import TransferCreateParams as TransferCreateParams
from .transfer_update_params import TransferUpdateParams as TransferUpdateParams
from .inventory_assembly_item import InventoryAssemblyItem as InventoryAssemblyItem
from .sales_order_list_params import SalesOrderListParams as SalesOrderListParams
from .bill_credit_card_payment import BillCreditCardPayment as BillCreditCardPayment
from .service_item_list_params import ServiceItemListParams as ServiceItemListParams
from .journal_entry_list_params import JournalEntryListParams as JournalEntryListParams
from .sales_order_create_params import SalesOrderCreateParams as SalesOrderCreateParams
from .sales_order_update_params import SalesOrderUpdateParams as SalesOrderUpdateParams
from .sales_receipt_list_params import SalesReceiptListParams as SalesReceiptListParams
from .standard_term_list_params import StandardTermListParams as StandardTermListParams
from .inventory_item_list_params import InventoryItemListParams as InventoryItemListParams
from .inventory_site_list_params import InventorySiteListParams as InventorySiteListParams
from .purchase_order_list_params import PurchaseOrderListParams as PurchaseOrderListParams
from .sales_tax_code_list_params import SalesTaxCodeListParams as SalesTaxCodeListParams
from .sales_tax_item_list_params import SalesTaxItemListParams as SalesTaxItemListParams
from .service_item_create_params import ServiceItemCreateParams as ServiceItemCreateParams
from .service_item_update_params import ServiceItemUpdateParams as ServiceItemUpdateParams
from .journal_entry_create_params import JournalEntryCreateParams as JournalEntryCreateParams
from .journal_entry_update_params import JournalEntryUpdateParams as JournalEntryUpdateParams
from .receive_payment_list_params import ReceivePaymentListParams as ReceivePaymentListParams
from .sales_receipt_create_params import SalesReceiptCreateParams as SalesReceiptCreateParams
from .sales_receipt_update_params import SalesReceiptUpdateParams as SalesReceiptUpdateParams
from .standard_term_create_params import StandardTermCreateParams as StandardTermCreateParams
from .standard_term_list_response import StandardTermListResponse as StandardTermListResponse
from .date_driven_term_list_params import DateDrivenTermListParams as DateDrivenTermListParams
from .inventory_item_create_params import InventoryItemCreateParams as InventoryItemCreateParams
from .inventory_item_update_params import InventoryItemUpdateParams as InventoryItemUpdateParams
from .inventory_site_create_params import InventorySiteCreateParams as InventorySiteCreateParams
from .inventory_site_list_response import InventorySiteListResponse as InventorySiteListResponse
from .inventory_site_update_params import InventorySiteUpdateParams as InventorySiteUpdateParams
from .purchase_order_create_params import PurchaseOrderCreateParams as PurchaseOrderCreateParams
from .purchase_order_update_params import PurchaseOrderUpdateParams as PurchaseOrderUpdateParams
from .sales_tax_code_create_params import SalesTaxCodeCreateParams as SalesTaxCodeCreateParams
from .sales_tax_code_list_response import SalesTaxCodeListResponse as SalesTaxCodeListResponse
from .sales_tax_code_update_params import SalesTaxCodeUpdateParams as SalesTaxCodeUpdateParams
from .sales_tax_item_create_params import SalesTaxItemCreateParams as SalesTaxItemCreateParams
from .sales_tax_item_update_params import SalesTaxItemUpdateParams as SalesTaxItemUpdateParams
from .receive_payment_create_params import ReceivePaymentCreateParams as ReceivePaymentCreateParams
from .receive_payment_update_params import ReceivePaymentUpdateParams as ReceivePaymentUpdateParams
from .bill_check_payment_list_params import BillCheckPaymentListParams as BillCheckPaymentListParams
from .credit_card_charge_list_params import CreditCardChargeListParams as CreditCardChargeListParams
from .credit_card_credit_list_params import CreditCardCreditListParams as CreditCardCreditListParams
from .date_driven_term_create_params import DateDrivenTermCreateParams as DateDrivenTermCreateParams
from .date_driven_term_list_response import DateDrivenTermListResponse as DateDrivenTermListResponse
from .non_inventory_item_list_params import NonInventoryItemListParams as NonInventoryItemListParams
from .bill_check_payment_create_params import BillCheckPaymentCreateParams as BillCheckPaymentCreateParams
from .bill_check_payment_update_params import BillCheckPaymentUpdateParams as BillCheckPaymentUpdateParams
from .credit_card_charge_create_params import CreditCardChargeCreateParams as CreditCardChargeCreateParams
from .credit_card_charge_update_params import CreditCardChargeUpdateParams as CreditCardChargeUpdateParams
from .credit_card_credit_create_params import CreditCardCreditCreateParams as CreditCardCreditCreateParams
from .credit_card_credit_update_params import CreditCardCreditUpdateParams as CreditCardCreditUpdateParams
from .non_inventory_item_create_params import NonInventoryItemCreateParams as NonInventoryItemCreateParams
from .non_inventory_item_update_params import NonInventoryItemUpdateParams as NonInventoryItemUpdateParams
from .inventory_assembly_item_list_params import InventoryAssemblyItemListParams as InventoryAssemblyItemListParams
from .bill_credit_card_payment_list_params import BillCreditCardPaymentListParams as BillCreditCardPaymentListParams
from .inventory_assembly_item_create_params import (
    InventoryAssemblyItemCreateParams as InventoryAssemblyItemCreateParams,
)
from .inventory_assembly_item_update_params import (
    InventoryAssemblyItemUpdateParams as InventoryAssemblyItemUpdateParams,
)
from .bill_credit_card_payment_create_params import (
    BillCreditCardPaymentCreateParams as BillCreditCardPaymentCreateParams,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Iterable
from datetime import date

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...types.qbd import (
    credit_card_credit_list_params,
    credit_card_credit_create_params,
    credit_card_credit_update_params,
)
from ...pagination import SyncCursorPage, AsyncCursorPage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.qbd.credit_card_credit import CreditCardCredit

__all__ = ["CreditCardCreditsResource", "AsyncCreditCardCreditsResource"]


class CreditCardCreditsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CreditCardCreditsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/conductor-is/conductor-python#accessing-raw-response-data-eg-headers
        """
        return CreditCardCreditsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CreditCardCreditsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/conductor-is/conductor-python#with_streaming_response
        """
        return CreditCardCreditsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        transaction_date: Union[str, date],
        conductor_end_user_id: str,
        exchange_rate: float | NotGiven = NOT_GIVEN,
        expense_lines: Iterable[credit_card_credit_create_params.ExpenseLine] | NotGiven = NOT_GIVEN,
        external_id: str | NotGiven = NOT_GIVEN,
        item_group_lines: Iterable[credit_card_credit_create_params.ItemGroupLine] | NotGiven = NOT_GIVEN,
        item_lines: Iterable[credit_card_credit_create_params.ItemLine] | NotGiven = NOT_GIVEN,
        memo: str | NotGiven = NOT_GIVEN,
        payee_id: str | NotGiven = NOT_GIVEN,
        ref_number: str | NotGiven = NOT_GIVEN,
        sales_tax_code_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CreditCardCredit:
        """
        Creates a new credit card credit for the specified account.

        Args:
          account_id: The bank or credit card account to which this credit card credit is applied.

          transaction_date: The date of this credit card credit, in ISO 8601 format (YYYY-MM-DD).

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          exchange_rate: The market exchange rate between this credit card credit's currency and the home
              currency in QuickBooks at the time of this transaction. Represented as a decimal
              value (e.g., 1.2345 for 1 EUR = 1.2345 USD if USD is the home currency).

          expense_lines: The credit card credit's expense lines, each representing one line in this
              expense.

          external_id: A globally unique identifier (GUID) you can provide for tracking this object in
              your external system.

              **IMPORTANT**: Must be formatted as a valid GUID; otherwise, QuickBooks will
              return an error. This field is immutable and can only be set during object
              creation.

          item_group_lines: The credit card credit's item group lines, each representing a predefined set of
              items bundled together because they are commonly purchased together or grouped
              for faster entry.

          item_lines: The credit card credit's item lines, each representing the purchase of a
              specific item or service.

          memo: A memo or note for this credit card credit.

          payee_id: The vendor or company from whom this credit card credit was received for
              purchased merchandise or services.

          ref_number: The case-sensitive user-defined reference number for this credit card credit,
              which can be used to identify the transaction in QuickBooks. This value is not
              required to be unique and can be arbitrarily changed by the QuickBooks user.

          sales_tax_code_id: The sales-tax code for this credit card credit, determining whether it is
              taxable or non-taxable. If set, this overrides any sales-tax codes defined on
              the payee. This can be overridden on the credit card credit's individual lines.

              Default codes include "Non" (non-taxable) and "Tax" (taxable), but custom codes
              can also be created in QuickBooks. If QuickBooks is not set up to charge sales
              tax (via the "Do You Charge Sales Tax?" preference), it will assign the default
              non-taxable code to all sales.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._post(
            "/quickbooks-desktop/credit-card-credits",
            body=maybe_transform(
                {
                    "account_id": account_id,
                    "transaction_date": transaction_date,
                    "exchange_rate": exchange_rate,
                    "expense_lines": expense_lines,
                    "external_id": external_id,
                    "item_group_lines": item_group_lines,
                    "item_lines": item_lines,
                    "memo": memo,
                    "payee_id": payee_id,
                    "ref_number": ref_number,
                    "sales_tax_code_id": sales_tax_code_id,
                },
                credit_card_credit_create_params.CreditCardCreditCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreditCardCredit,
        )

    def retrieve(
        self,
        id: str,
        *,
        conductor_end_user_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CreditCardCredit:
        """
        Retrieves a credit card credit by ID.

        Args:
          id: The QuickBooks-assigned unique identifier of the credit card credit to retrieve.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._get(
            f"/quickbooks-desktop/credit-card-credits/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreditCardCredit,
        )

    def update(
        self,
        id: str,
        *,
        revision_number: str,
        conductor_end_user_id: str,
        account_id: str | NotGiven = NOT_GIVEN,
        clear_expense_lines: bool | NotGiven = NOT_GIVEN,
        clear_item_lines: bool | NotGiven = NOT_GIVEN,
        exchange_rate: float | NotGiven = NOT_GIVEN,
        expense_lines: Iterable[credit_card_credit_update_params.ExpenseLine] | NotGiven = NOT_GIVEN,
        item_group_lines: Iterable[credit_card_credit_update_params.ItemGroupLine] | NotGiven = NOT_GIVEN,
        item_lines: Iterable[credit_card_credit_update_params.ItemLine] | NotGiven = NOT_GIVEN,
        memo: str | NotGiven = NOT_GIVEN,
        payee_id: str | NotGiven = NOT_GIVEN,
        ref_number: str | NotGiven = NOT_GIVEN,
        sales_tax_code_id: str | NotGiven = NOT_GIVEN,
        transaction_date: Union[str, date] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CreditCardCredit:
        """
        Updates an existing credit card credit.

        Args:
          id: The QuickBooks-assigned unique identifier of the credit card credit to update.

          revision_number: The current revision number of the credit card credit object you are updating,
              which you can get by fetching the object first. Provide the most recent
              `revisionNumber` to ensure you're working with the latest data; otherwise, the
              update will return an error.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          account_id: The bank or credit card account to which this credit card credit is applied.

          clear_expense_lines: When `true`, removes all existing expense lines associated with this credit card
              credit. To modify or add individual expense lines, use the field `expenseLines`
              instead.

          clear_item_lines: When `true`, removes all existing item lines associated with this credit card
              credit. To modify or add individual item lines, use the field `itemLines`
              instead.

          exchange_rate: The market exchange rate between this credit card credit's currency and the home
              currency in QuickBooks at the time of this transaction. Represented as a decimal
              value (e.g., 1.2345 for 1 EUR = 1.2345 USD if USD is the home currency).

          expense_lines: The credit card credit's expense lines, each representing one line in this
              expense.

              **IMPORTANT**: When updating a credit card credit's expense lines, this array
              completely REPLACES all existing expense lines for that credit card credit. To
              retain any current expense lines, include them in this array, even if they have
              not changed. Any expense lines not included will be removed. To add a new
              expense line, include it with its `id` set to `-1`. If you do not wish to modify
              the expense lines, you can omit this field entirely to keep them unchanged.

          item_group_lines: The credit card credit's item group lines, each representing a predefined set of
              items bundled together because they are commonly purchased together or grouped
              for faster entry.

              **IMPORTANT**: When updating a credit card credit's item group lines, this array
              completely REPLACES all existing item group lines for that credit card credit.
              To retain any current item group lines, include them in this array, even if they
              have not changed. Any item group lines not included will be removed. To add a
              new item group line, include it with its `id` set to `-1`. If you do not wish to
              modify the item group lines, you can omit this field entirely to keep them
              unchanged.

          item_lines: The credit card credit's item lines, each representing the purchase of a
              specific item or service.

              **IMPORTANT**: When updating a credit card credit's item lines, this array
              completely REPLACES all existing item lines for that credit card credit. To
              retain any current item lines, include them in this array, even if they have not
              changed. Any item lines not included will be removed. To add a new item line,
              include it with its `id` set to `-1`. If you do not wish to modify the item
              lines, you can omit this field entirely to keep them unchanged.

          memo: A memo or note for this credit card credit.

          payee_id: The vendor or company from whom this credit card credit was received for
              purchased merchandise or services.

          ref_number: The case-sensitive user-defined reference number for this credit card credit,
              which can be used to identify the transaction in QuickBooks. This value is not
              required to be unique and can be arbitrarily changed by the QuickBooks user.

          sales_tax_code_id: The sales-tax code for this credit card credit, determining whether it is
              taxable or non-taxable. If set, this overrides any sales-tax codes defined on
              the payee. This can be overridden on the credit card credit's individual lines.

              Default codes include "Non" (non-taxable) and "Tax" (taxable), but custom codes
              can also be created in QuickBooks. If QuickBooks is not set up to charge sales
              tax (via the "Do You Charge Sales Tax?" preference), it will assign the default
              non-taxable code to all sales.

          transaction_date: The date of this credit card credit, in ISO 8601 format (YYYY-MM-DD).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._post(
            f"/quickbooks-desktop/credit-card-credits/{id}",
            body=maybe_transform(
                {
                    "revision_number": revision_number,
                    "account_id": account_id,
                    "clear_expense_lines": clear_expense_lines,
                    "clear_item_lines": clear_item_lines,
                    "exchange_rate": exchange_rate,
                    "expense_lines": expense_lines,
                    "item_group_lines": item_group_lines,
                    "item_lines": item_lines,
                    "memo": memo,
                    "payee_id": payee_id,
                    "ref_number": ref_number,
                    "sales_tax_code_id": sales_tax_code_id,
                    "transaction_date": transaction_date,
                },
                credit_card_credit_update_params.CreditCardCreditUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreditCardCredit,
        )

    def list(
        self,
        *,
        conductor_end_user_id: str,
        account_ids: List[str] | NotGiven = NOT_GIVEN,
        currency_ids: List[str] | NotGiven = NOT_GIVEN,
        cursor: str | NotGiven = NOT_GIVEN,
        ids: List[str] | NotGiven = NOT_GIVEN,
        include_line_items: bool | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        payee_ids: List[str] | NotGiven = NOT_GIVEN,
        ref_number_contains: str | NotGiven = NOT_GIVEN,
        ref_number_ends_with: str | NotGiven = NOT_GIVEN,
        ref_number_from: str | NotGiven = NOT_GIVEN,
        ref_numbers: List[str] | NotGiven = NOT_GIVEN,
        ref_number_starts_with: str | NotGiven = NOT_GIVEN,
        ref_number_to: str | NotGiven = NOT_GIVEN,
        transaction_date_from: Union[str, date] | NotGiven = NOT_GIVEN,
        transaction_date_to: Union[str, date] | NotGiven = NOT_GIVEN,
        updated_after: str | NotGiven = NOT_GIVEN,
        updated_before: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncCursorPage[CreditCardCredit]:
        """Returns a list of credit card credits.

        Use the `cursor` parameter to paginate
        through the results.

        Args:
          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          account_ids: Filter for credit card credits associated with these accounts.

          currency_ids: Filter for credit card credits in these currencies.

          cursor: The pagination token to fetch the next set of results when paginating with the
              `limit` parameter. Retrieve this value from the `nextCursor` field in the
              previous response. If omitted, the API returns the first page of results.

          ids: Filter for specific credit card credits by their QuickBooks-assigned unique
              identifier(s).

              **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
              query parameters for this request.

          include_line_items: Whether to include line items in the response. Defaults to `true`.

          limit: The maximum number of objects to return. Accepts values ranging from 1 to 150,
              defaults to 150. When used with cursor-based pagination, this parameter controls
              how many results are returned per page. To paginate through results, combine
              this with the `cursor` parameter. Each response will include a `nextCursor`
              value that can be passed to subsequent requests to retrieve the next page of
              results.

          payee_ids: Filter for credit card credits received from these payees. These are the vendors
              or companies from whom these credit card credits were received.

          ref_number_contains:
              Filter for credit card credits whose `refNumber` contains this substring. NOTE:
              If you use this parameter, you cannot also use `refNumberStartsWith` or
              `refNumberEndsWith`.

          ref_number_ends_with:
              Filter for credit card credits whose `refNumber` ends with this substring. NOTE:
              If you use this parameter, you cannot also use `refNumberContains` or
              `refNumberStartsWith`.

          ref_number_from: Filter for credit card credits whose `refNumber` is greater than or equal to
              this value. If omitted, the range will begin with the first number of the list.
              Uses a numerical comparison for values that contain only digits; otherwise, uses
              a lexicographical comparison.

          ref_numbers: Filter for specific credit card credits by their ref-number(s), case-sensitive.
              In QuickBooks, ref-numbers are not required to be unique and can be arbitrarily
              changed by the QuickBooks user.

              **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
              query parameters for this request.

          ref_number_starts_with: Filter for credit card credits whose `refNumber` starts with this substring.
              NOTE: If you use this parameter, you cannot also use `refNumberContains` or
              `refNumberEndsWith`.

          ref_number_to: Filter for credit card credits whose `refNumber` is less than or equal to this
              value. If omitted, the range will end with the last number of the list. Uses a
              numerical comparison for values that contain only digits; otherwise, uses a
              lexicographical comparison.

          transaction_date_from: Filter for credit card credits created on or after this date, in ISO 8601 format
              (YYYY-MM-DD).

          transaction_date_to: Filter for credit card credits created on or before this date, in ISO 8601
              format (YYYY-MM-DD).

          updated_after: Filter for credit card credits updated on or after this date and time, in ISO
              8601 format (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD), the
              time is assumed to be 00:00:00 of that day.

          updated_before: Filter for credit card credits updated on or before this date and time, in ISO
              8601 format (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD), the
              time is assumed to be 23:59:59 of that day.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._get_api_list(
            "/quickbooks-desktop/credit-card-credits",
            page=SyncCursorPage[CreditCardCredit],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_ids": account_ids,
                        "currency_ids": currency_ids,
                        "cursor": cursor,
                        "ids": ids,
                        "include_line_items": include_line_items,
                        "limit": limit,
                        "payee_ids": payee_ids,
                        "ref_number_contains": ref_number_contains,
                        "ref_number_ends_with": ref_number_ends_with,
                        "ref_number_from": ref_number_from,
                        "ref_numbers": ref_numbers,
                        "ref_number_starts_with": ref_number_starts_with,
                        "ref_number_to": ref_number_to,
                        "transaction_date_from": transaction_date_from,
                        "transaction_date_to": transaction_date_to,
                        "updated_after": updated_after,
                        "updated_before": updated_before,
                    },
                    credit_card_credit_list_params.CreditCardCreditListParams,
                ),
            ),
            model=CreditCardCredit,
        )


class AsyncCreditCardCreditsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCreditCardCreditsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/conductor-is/conductor-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCreditCardCreditsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCreditCardCreditsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/conductor-is/conductor-python#with_streaming_response
        """
        return AsyncCreditCardCreditsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        transaction_date: Union[str, date],
        conductor_end_user_id: str,
        exchange_rate: float | NotGiven = NOT_GIVEN,
        expense_lines: Iterable[credit_card_credit_create_params.ExpenseLine] | NotGiven = NOT_GIVEN,
        external_id: str | NotGiven = NOT_GIVEN,
        item_group_lines: Iterable[credit_card_credit_create_params.ItemGroupLine] | NotGiven = NOT_GIVEN,
        item_lines: Iterable[credit_card_credit_create_params.ItemLine] | NotGiven = NOT_GIVEN,
        memo: str | NotGiven = NOT_GIVEN,
        payee_id: str | NotGiven = NOT_GIVEN,
        ref_number: str | NotGiven = NOT_GIVEN,
        sales_tax_code_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CreditCardCredit:
        """
        Creates a new credit card credit for the specified account.

        Args:
          account_id: The bank or credit card account to which this credit card credit is applied.

          transaction_date: The date of this credit card credit, in ISO 8601 format (YYYY-MM-DD).

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          exchange_rate: The market exchange rate between this credit card credit's currency and the home
              currency in QuickBooks at the time of this transaction. Represented as a decimal
              value (e.g., 1.2345 for 1 EUR = 1.2345 USD if USD is the home currency).

          expense_lines: The credit card credit's expense lines, each representing one line in this
              expense.

          external_id: A globally unique identifier (GUID) you can provide for tracking this object in
              your external system.

              **IMPORTANT**: Must be formatted as a valid GUID; otherwise, QuickBooks will
              return an error. This field is immutable and can only be set during object
              creation.

          item_group_lines: The credit card credit's item group lines, each representing a predefined set of
              items bundled together because they are commonly purchased together or grouped
              for faster entry.

          item_lines: The credit card credit's item lines, each representing the purchase of a
              specific item or service.

          memo: A memo or note for this credit card credit.

          payee_id: The vendor or company from whom this credit card credit was received for
              purchased merchandise or services.

          ref_number: The case-sensitive user-defined reference number for this credit card credit,
              which can be used to identify the transaction in QuickBooks. This value is not
              required to be unique and can be arbitrarily changed by the QuickBooks user.

          sales_tax_code_id: The sales-tax code for this credit card credit, determining whether it is
              taxable or non-taxable. If set, this overrides any sales-tax codes defined on
              the payee. This can be overridden on the credit card credit's individual lines.

              Default codes include "Non" (non-taxable) and "Tax" (taxable), but custom codes
              can also be created in QuickBooks. If QuickBooks is not set up to charge sales
              tax (via the "Do You Charge Sales Tax?" preference), it will assign the default
              non-taxable code to all sales.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return await self._post(
            "/quickbooks-desktop/credit-card-credits",
            body=await async_maybe_transform(
                {
                    "account_id": account_id,
                    "transaction_date": transaction_date,
                    "exchange_rate": exchange_rate,
                    "expense_lines": expense_lines,
                    "external_id": external_id,
                    "item_group_lines": item_group_lines,
                    "item_lines": item_lines,
                    "memo": memo,
                    "payee_id": payee_id,
                    "ref_number": ref_number,
                    "sales_tax_code_id": sales_tax_code_id,
                },
                credit_card_credit_create_params.CreditCardCreditCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreditCardCredit,
        )

    async def retrieve(
        self,
        id: str,
        *,
        conductor_end_user_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CreditCardCredit:
        """
        Retrieves a credit card credit by ID.

        Args:
          id: The QuickBooks-assigned unique identifier of the credit card credit to retrieve.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return await self._get(
            f"/quickbooks-desktop/credit-card-credits/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreditCardCredit,
        )

    async def update(
        self,
        id: str,
        *,
        revision_number: str,
        conductor_end_user_id: str,
        account_id: str | NotGiven = NOT_GIVEN,
        clear_expense_lines: bool | NotGiven = NOT_GIVEN,
        clear_item_lines: bool | NotGiven = NOT_GIVEN,
        exchange_rate: float | NotGiven = NOT_GIVEN,
        expense_lines: Iterable[credit_card_credit_update_params.ExpenseLine] | NotGiven = NOT_GIVEN,
        item_group_lines: Iterable[credit_card_credit_update_params.ItemGroupLine] | NotGiven = NOT_GIVEN,
        item_lines: Iterable[credit_card_credit_update_params.ItemLine] | NotGiven = NOT_GIVEN,
        memo: str | NotGiven = NOT_GIVEN,
        payee_id: str | NotGiven = NOT_GIVEN,
        ref_number: str | NotGiven = NOT_GIVEN,
        sales_tax_code_id: str | NotGiven = NOT_GIVEN,
        transaction_date: Union[str, date] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CreditCardCredit:
        """
        Updates an existing credit card credit.

        Args:
          id: The QuickBooks-assigned unique identifier of the credit card credit to update.

          revision_number: The current revision number of the credit card credit object you are updating,
              which you can get by fetching the object first. Provide the most recent
              `revisionNumber` to ensure you're working with the latest data; otherwise, the
              update will return an error.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          account_id: The bank or credit card account to which this credit card credit is applied.

          clear_expense_lines: When `true`, removes all existing expense lines associated with this credit card
              credit. To modify or add individual expense lines, use the field `expenseLines`
              instead.

          clear_item_lines: When `true`, removes all existing item lines associated with this credit card
              credit. To modify or add individual item lines, use the field `itemLines`
              instead.

          exchange_rate: The market exchange rate between this credit card credit's currency and the home
              currency in QuickBooks at the time of this transaction. Represented as a decimal
              value (e.g., 1.2345 for 1 EUR = 1.2345 USD if USD is the home currency).

          expense_lines: The credit card credit's expense lines, each representing one line in this
              expense.

              **IMPORTANT**: When updating a credit card credit's expense lines, this array
              completely REPLACES all existing expense lines for that credit card credit. To
              retain any current expense lines, include them in this array, even if they have
              not changed. Any expense lines not included will be removed. To add a new
              expense line, include it with its `id` set to `-1`. If you do not wish to modify
              the expense lines, you can omit this field entirely to keep them unchanged.

          item_group_lines: The credit card credit's item group lines, each representing a predefined set of
              items bundled together because they are commonly purchased together or grouped
              for faster entry.

              **IMPORTANT**: When updating a credit card credit's item group lines, this array
              completely REPLACES all existing item group lines for that credit card credit.
              To retain any current item group lines, include them in this array, even if they
              have not changed. Any item group lines not included will be removed. To add a
              new item group line, include it with its `id` set to `-1`. If you do not wish to
              modify the item group lines, you can omit this field entirely to keep them
              unchanged.

          item_lines: The credit card credit's item lines, each representing the purchase of a
              specific item or service.

              **IMPORTANT**: When updating a credit card credit's item lines, this array
              completely REPLACES all existing item lines for that credit card credit. To
              retain any current item lines, include them in this array, even if they have not
              changed. Any item lines not included will be removed. To add a new item line,
              include it with its `id` set to `-1`. If you do not wish to modify the item
              lines, you can omit this field entirely to keep them unchanged.

          memo: A memo or note for this credit card credit.

          payee_id: The vendor or company from whom this credit card credit was received for
              purchased merchandise or services.

          ref_number: The case-sensitive user-defined reference number for this credit card credit,
              which can be used to identify the transaction in QuickBooks. This value is not
              required to be unique and can be arbitrarily changed by the QuickBooks user.

          sales_tax_code_id: The sales-tax code for this credit card credit, determining whether it is
              taxable or non-taxable. If set, this overrides any sales-tax codes defined on
              the payee. This can be overridden on the credit card credit's individual lines.

              Default codes include "Non" (non-taxable) and "Tax" (taxable), but custom codes
              can also be created in QuickBooks. If QuickBooks is not set up to charge sales
              tax (via the "Do You Charge Sales Tax?" preference), it will assign the default
              non-taxable code to all sales.

          transaction_date: The date of this credit card credit, in ISO 8601 format (YYYY-MM-DD).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return await self._post(
            f"/quickbooks-desktop/credit-card-credits/{id}",
            body=await async_maybe_transform(
                {
                    "revision_number": revision_number,
                    "account_id": account_id,
                    "clear_expense_lines": clear_expense_lines,
                    "clear_item_lines": clear_item_lines,
                    "exchange_rate": exchange_rate,
                    "expense_lines": expense_lines,
                    "item_group_lines": item_group_lines,
                    "item_lines": item_lines,
                    "memo": memo,
                    "payee_id": payee_id,
                    "ref_number": ref_number,
                    "sales_tax_code_id": sales_tax_code_id,
                    "transaction_date": transaction_date,
                },
                credit_card_credit_update_params.CreditCardCreditUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreditCardCredit,
        )

    def list(
        self,
        *,
        conductor_end_user_id: str,
        account_ids: List[str] | NotGiven = NOT_GIVEN,
        currency_ids: List[str] | NotGiven = NOT_GIVEN,
        cursor: str | NotGiven = NOT_GIVEN,
        ids: List[str] | NotGiven = NOT_GIVEN,
        include_line_items: bool | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        payee_ids: List[str] | NotGiven = NOT_GIVEN,
        ref_number_contains: str | NotGiven = NOT_GIVEN,
        ref_number_ends_with: str | NotGiven = NOT_GIVEN,
        ref_number_from: str | NotGiven = NOT_GIVEN,
        ref_numbers: List[str] | NotGiven = NOT_GIVEN,
        ref_number_starts_with: str | NotGiven = NOT_GIVEN,
        ref_number_to: str | NotGiven = NOT_GIVEN,
        transaction_date_from: Union[str, date] | NotGiven = NOT_GIVEN,
        transaction_date_to: Union[str, date] | NotGiven = NOT_GIVEN,
        updated_after: str | NotGiven = NOT_GIVEN,
        updated_before: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[CreditCardCredit, AsyncCursorPage[CreditCardCredit]]:
        """Returns a list of credit card credits.

        Use the `cursor` parameter to paginate
        through the results.

        Args:
          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          account_ids: Filter for credit card credits associated with these accounts.

          currency_ids: Filter for credit card credits in these currencies.

          cursor: The pagination token to fetch the next set of results when paginating with the
              `limit` parameter. Retrieve this value from the `nextCursor` field in the
              previous response. If omitted, the API returns the first page of results.

          ids: Filter for specific credit card credits by their QuickBooks-assigned unique
              identifier(s).

              **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
              query parameters for this request.

          include_line_items: Whether to include line items in the response. Defaults to `true`.

          limit: The maximum number of objects to return. Accepts values ranging from 1 to 150,
              defaults to 150. When used with cursor-based pagination, this parameter controls
              how many results are returned per page. To paginate through results, combine
              this with the `cursor` parameter. Each response will include a `nextCursor`
              value that can be passed to subsequent requests to retrieve the next page of
              results.

          payee_ids: Filter for credit card credits received from these payees. These are the vendors
              or companies from whom these credit card credits were received.

          ref_number_contains:
              Filter for credit card credits whose `refNumber` contains this substring. NOTE:
              If you use this parameter, you cannot also use `refNumberStartsWith` or
              `refNumberEndsWith`.

          ref_number_ends_with:
              Filter for credit card credits whose `refNumber` ends with this substring. NOTE:
              If you use this parameter, you cannot also use `refNumberContains` or
              `refNumberStartsWith`.

          ref_number_from: Filter for credit card credits whose `refNumber` is greater than or equal to
              this value. If omitted, the range will begin with the first number of the list.
              Uses a numerical comparison for values that contain only digits; otherwise, uses
              a lexicographical comparison.

          ref_numbers: Filter for specific credit card credits by their ref-number(s), case-sensitive.
              In QuickBooks, ref-numbers are not required to be unique and can be arbitrarily
              changed by the QuickBooks user.

              **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
              query parameters for this request.

          ref_number_starts_with: Filter for credit card credits whose `refNumber` starts with this substring.
              NOTE: If you use this parameter, you cannot also use `refNumberContains` or
              `refNumberEndsWith`.

          ref_number_to: Filter for credit card credits whose `refNumber` is less than or equal to this
              value. If omitted, the range will end with the last number of the list. Uses a
              numerical comparison for values that contain only digits; otherwise, uses a
              lexicographical comparison.

          transaction_date_from: Filter for credit card credits created on or after this date, in ISO 8601 format
              (YYYY-MM-DD).

          transaction_date_to: Filter for credit card credits created on or before this date, in ISO 8601
              format (YYYY-MM-DD).

          updated_after: Filter for credit card credits updated on or after this date and time, in ISO
              8601 format (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD), the
              time is assumed to be 00:00:00 of that day.

          updated_before: Filter for credit card credits updated on or before this date and time, in ISO
              8601 format (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD), the
              time is assumed to be 23:59:59 of that day.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._get_api_list(
            "/quickbooks-desktop/credit-card-credits",
            page=AsyncCursorPage[CreditCardCredit],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_ids": account_ids,
                        "currency_ids": currency_ids,
                        "cursor": cursor,
                        "ids": ids,
                        "include_line_items": include_line_items,
                        "limit": limit,
                        "payee_ids": payee_ids,
                        "ref_number_contains": ref_number_contains,
                        "ref_number_ends_with": ref_number_ends_with,
                        "ref_number_from": ref_number_from,
                        "ref_numbers": ref_numbers,
                        "ref_number_starts_with": ref_number_starts_with,
                        "ref_number_to": ref_number_to,
                        "transaction_date_from": transaction_date_from,
                        "transaction_date_to": transaction_date_to,
                        "updated_after": updated_after,
                        "updated_before": updated_before,
                    },
                    credit_card_credit_list_params.CreditCardCreditListParams,
                ),
            ),
            model=CreditCardCredit,
        )


class CreditCardCreditsResourceWithRawResponse:
    def __init__(self, credit_card_credits: CreditCardCreditsResource) -> None:
        self._credit_card_credits = credit_card_credits

        self.create = to_raw_response_wrapper(
            credit_card_credits.create,
        )
        self.retrieve = to_raw_response_wrapper(
            credit_card_credits.retrieve,
        )
        self.update = to_raw_response_wrapper(
            credit_card_credits.update,
        )
        self.list = to_raw_response_wrapper(
            credit_card_credits.list,
        )


class AsyncCreditCardCreditsResourceWithRawResponse:
    def __init__(self, credit_card_credits: AsyncCreditCardCreditsResource) -> None:
        self._credit_card_credits = credit_card_credits

        self.create = async_to_raw_response_wrapper(
            credit_card_credits.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            credit_card_credits.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            credit_card_credits.update,
        )
        self.list = async_to_raw_response_wrapper(
            credit_card_credits.list,
        )


class CreditCardCreditsResourceWithStreamingResponse:
    def __init__(self, credit_card_credits: CreditCardCreditsResource) -> None:
        self._credit_card_credits = credit_card_credits

        self.create = to_streamed_response_wrapper(
            credit_card_credits.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            credit_card_credits.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            credit_card_credits.update,
        )
        self.list = to_streamed_response_wrapper(
            credit_card_credits.list,
        )


class AsyncCreditCardCreditsResourceWithStreamingResponse:
    def __init__(self, credit_card_credits: AsyncCreditCardCreditsResource) -> None:
        self._credit_card_credits = credit_card_credits

        self.create = async_to_streamed_response_wrapper(
            credit_card_credits.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            credit_card_credits.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            credit_card_credits.update,
        )
        self.list = async_to_streamed_response_wrapper(
            credit_card_credits.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from conductor import Conductor, AsyncConductor
from tests.utils import assert_matches_type
from conductor._utils import parse_date
from conductor.types.qbd import (
    InventoryItem,
)
from conductor.pagination import SyncCursorPage, AsyncCursorPage

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestInventoryItems:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: Conductor) -> None:
        inventory_item = client.qbd.inventory_items.create(
            asset_account_id="80000009-1234567890",
            cogs_account_id="80000007-1234567890",
            income_account_id="80000005-1234567890",
            name="Cabinet",
            conductor_end_user_id="end_usr_1234567abcdefg",
        )
        assert_matches_type(InventoryItem, inventory_item, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: Conductor) -> None:
        inventory_item = client.qbd.inventory_items.create(
            asset_account_id="80000009-1234567890",
            cogs_account_id="80000007-1234567890",
            income_account_id="80000005-1234567890",
            name="Cabinet",
            conductor_end_user_id="end_usr_1234567abcdefg",
            barcode={
                "allow_override": False,
                "assign_even_if_used": False,
                "value": "012345678905",
            },
            class_id="80000001-1234567890",
            external_id="12345678-abcd-1234-abcd-1234567890ab",
            inventory_date=parse_date("2019-12-27"),
            is_active=True,
            maximum_quantity_on_hand=200,
            parent_id="80000002-1234567890",
            preferred_vendor_id="80000008-1234567890",
            purchase_cost="15.75",
            purchase_description="Bulk purchase of steel bolts for inventory",
            purchase_tax_code_id="80000006-1234567890",
            quantity_on_hand=150,
            reorder_point=50,
            sales_description="High-quality steel bolts suitable for construction",
            sales_price="19.99",
            sales_tax_code_id="80000004-1234567890",
            sku="MPN-123456",
            total_value="1500.00",
            unit_of_measure_set_id="80000003-1234567890",
        )
        assert_matches_type(InventoryItem, inventory_item, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: Conductor) -> None:
        response = client.qbd.inventory_items.with_raw_response.create(
            asset_account_id="80000009-1234567890",
            cogs_account_id="80000007-1234567890",
            income_account_id="80000005-1234567890",
            name="Cabinet",
            conductor_end_user_id="end_usr_1234567abcdefg",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        inventory_item = response.parse()
        assert_matches_type(InventoryItem, inventory_item, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: Conductor) -> None:
        with client.qbd.inventory_items.with_streaming_response.create(
            asset_account_id="80000009-1234567890",
            cogs_account_id="80000007-1234567890",
            income_account_id="80000005-1234567890",
            name="Cabinet",
            conductor_end_user_id="end_usr_1234567abcdefg",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            inventory_item = response.parse()
            assert_matches_type(InventoryItem, inventory_item, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: Conductor) -> None:
        inventory_item = client.qbd.inventory_items.retrieve(
            id="80000001-1234567890",
            conductor_end_user_id="end_usr_1234567abcdefg",
        )
        assert_matches_type(InventoryItem, inventory_item, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: Conductor) -> None:
        response = client.qbd.inventory_items.with_raw_response.retrieve(
            id="80000001-1234567890",
            conductor_end_user_id="end_usr_1234567abcdefg",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        inventory_item = response.parse()
        assert_matches_type(InventoryItem, inventory_item, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: Conductor) -> None:
        with client.qbd.inventory_items.with_streaming_response.retrieve(
            id="80000001-1234567890",
            conductor_end_user_id="end_usr_1234567abcdefg",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            inventory_item = response.parse()
            assert_matches_type(InventoryItem, inventory_item, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: Conductor) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.qbd.inventory_items.with_raw_response.retrieve(
                id="",
                conductor_end_user_id="end_usr_1234567abcdefg",
            )

    @parametrize
    def test_method_update(self, client: Conductor) -> None:
        inventory_item = client.qbd.inventory_items.update(
            id="80000001-1234567890",
            revision_number="1721172183",
            conductor_end_user_id="end_usr_1234567abcdefg",
        )
        assert_matches_type(InventoryItem, inventory_item, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: Conductor) -> None:
        inventory_item = client.qbd.inventory_items.update(
            id="80000001-1234567890",
            revision_number="1721172183",
            conductor_end_user_id="end_usr_1234567abcdefg",
            asset_account_id="80000009-1234567890",
            barcode={
                "allow_override": False,
                "assign_even_if_used": False,
                "value": "012345678905",
            },
            class_id="80000001-1234567890",
            cogs_account_id="80000007-1234567890",
            force_unit_of_measure_change=False,
            income_account_id="80000005-1234567890",
            is_active=True,
            maximum_quantity_on_hand=200,
            name="Cabinet",
            parent_id="80000002-1234567890",
            preferred_vendor_id="80000008-1234567890",
            purchase_cost="15.75",
            purchase_description="Bulk purchase of steel bolts for inventory",
            purchase_tax_code_id="80000006-1234567890",
            reorder_point=50,
            sales_description="High-quality steel bolts suitable for construction",
            sales_price="19.99",
            sales_tax_code_id="80000004-1234567890",
            sku="MPN-123456",
            unit_of_measure_set_id="80000003-1234567890",
            update_existing_transactions_cogs_account=False,
            update_existing_transactions_income_account=False,
        )
        assert_matches_type(InventoryItem, inventory_item, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: Conductor) -> None:
        response = client.qbd.inventory_items.with_raw_response.update(
            id="80000001-1234567890",
            revision_number="1721172183",
            conductor_end_user_id="end_usr_1234567abcdefg",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        inventory_item = response.parse()
        assert_matches_type(InventoryItem, inventory_item, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: Conductor) -> None:
        with client.qbd.inventory_items.with_streaming_response.update(
            id="80000001-1234567890",
            revision_number="1721172183",
            conductor_end_user_id="end_usr_1234567abcdefg",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            inventory_item = response.parse()
            assert_matches_type(InventoryItem, inventory_item, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: Conductor) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.qbd.inventory_items.with_raw_response.update(
                id="",
                revision_number="1721172183",
                conductor_end_user_id="end_usr_1234567abcdefg",
            )

    @parametrize
    def test_method_list(self, client: Conductor) -> None:
        inventory_item = client.qbd.inventory_items.list(
            conductor_end_user_id="end_usr_1234567abcdefg",
        )
        assert_matches_type(SyncCursorPage[InventoryItem], inventory_item, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: Conductor) -> None:
        inventory_item = client.qbd.inventory_items.list(
            conductor_end_user_id="end_usr_1234567abcdefg",
            class_ids=["80000001-1234567890"],
            cursor="12345678-abcd-abcd-example-1234567890ab",
            full_names=["Kitchen:Cabinet"],
            ids=["80000001-1234567890"],
            limit=150,
            name_contains="ABC",
            name_ends_with="ABC",
            name_from="A",
            name_starts_with="ABC",
            name_to="Z",
            status="active",
            updated_after="updatedAfter",
            updated_before="updatedBefore",
        )
        assert_matches_type(SyncCursorPage[InventoryItem], inventory_item, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Conductor) -> None:
        response = client.qbd.inventory_items.with_raw_response.list(
            conductor_end_user_id="end_usr_1234567abcdefg",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        inventory_item = response.parse()
        assert_matches_type(SyncCursorPage[InventoryItem], inventory_item, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Conductor) -> None:
        with client.qbd.inventory_items.with_streaming_response.list(
            conductor_end_user_id="end_usr_1234567abcdefg",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            inventory_item = response.parse()
            assert_matches_type(SyncCursorPage[InventoryItem], inventory_item, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncInventoryItems:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncConductor) -> None:
        inventory_item = await async_client.qbd.inventory_items.create(
            asset_account_id="80000009-1234567890",
            cogs_account_id="80000007-1234567890",
            income_account_id="80000005-1234567890",
            name="Cabinet",
            conductor_end_user_id="end_usr_1234567abcdefg",
        )
        assert_matches_type(InventoryItem, inventory_item, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncConductor) -> None:
        inventory_item = await async_client.qbd.inventory_items.create(
            asset_account_id="80000009-1234567890",
            cogs_account_id="80000007-1234567890",
            income_account_id="80000005-1234567890",
            name="Cabinet",
            conductor_end_user_id="end_usr_1234567abcdefg",
            barcode={
                "allow_override": False,
                "assign_even_if_used": False,
                "value": "012345678905",
            },
            class_id="80000001-1234567890",
            external_id="12345678-abcd-1234-abcd-1234567890ab",
            inventory_date=parse_date("2019-12-27"),
            is_active=True,
            maximum_quantity_on_hand=200,
            parent_id="80000002-1234567890",
            preferred_vendor_id="80000008-1234567890",
            purchase_cost="15.75",
            purchase_description="Bulk purchase of steel bolts for inventory",
            purchase_tax_code_id="80000006-1234567890",
            quantity_on_hand=150,
            reorder_point=50,
            sales_description="High-quality steel bolts suitable for construction",
            sales_price="19.99",
            sales_tax_code_id="80000004-1234567890",
            sku="MPN-123456",
            total_value="1500.00",
            unit_of_measure_set_id="80000003-1234567890",
        )
        assert_matches_type(InventoryItem, inventory_item, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncConductor) -> None:
        response = await async_client.qbd.inventory_items.with_raw_response.create(
            asset_account_id="80000009-1234567890",
            cogs_account_id="80000007-1234567890",
            income_account_id="80000005-1234567890",
            name="Cabinet",
            conductor_end_user_id="end_usr_1234567abcdefg",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        inventory_item = await response.parse()
        assert_matches_type(InventoryItem, inventory_item, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncConductor) -> None:
        async with async_client.qbd.inventory_items.with_streaming_response.create(
            asset_account_id="80000009-1234567890",
            cogs_account_id="80000007-1234567890",
            income_account_id="80000005-1234567890",
            name="Cabinet",
            conductor_end_user_id="end_usr_1234567abcdefg",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            inventory_item = await response.parse()
            assert_matches_type(InventoryItem, inventory_item, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncConductor) -> None:
        inventory_item = await async_client.qbd.inventory_items.retrieve(
            id="80000001-1234567890",
            conductor_end_user_id="end_usr_1234567abcdefg",
        )
        assert_matches_type(InventoryItem, inventory_item, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncConductor) -> None:
        response = await async_client.qbd.inventory_items.with_raw_response.retrieve(
            id="80000001-1234567890",
            conductor_end_user_id="end_usr_1234567abcdefg",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        inventory_item = await response.parse()
        assert_matches_type(InventoryItem, inventory_item, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncConductor) -> None:
        async with async_client.qbd.inventory_items.with_streaming_response.retrieve(
            id="80000001-1234567890",
            conductor_end_user_id="end_usr_1234567abcdefg",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            inventory_item = await response.parse()
            assert_matches_type(InventoryItem, inventory_item, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncConductor) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.qbd.inventory_items.with_raw_response.retrieve(
                id="",
                conductor_end_user_id="end_usr_1234567abcdefg",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncConductor) -> None:
        inventory_item = await async_client.qbd.inventory_items.update(
            id="80000001-1234567890",
            revision_number="1721172183",
            conductor_end_user_id="end_usr_1234567abcdefg",
        )
        assert_matches_type(InventoryItem, inventory_item, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncConductor) -> None:
        inventory_item = await async_client.qbd.inventory_items.update(
            id="80000001-1234567890",
            revision_number="1721172183",
            conductor_end_user_id="end_usr_1234567abcdefg",
            asset_account_id="80000009-1234567890",
            barcode={
                "allow_override": False,
                "assign_even_if_used": False,
                "value": "012345678905",
            },
            class_id="80000001-1234567890",
            cogs_account_id="80000007-1234567890",
            force_unit_of_measure_change=False,
            income_account_id="80000005-1234567890",
            is_active=True,
            maximum_quantity_on_hand=200,
            name="Cabinet",
            parent_id="80000002-1234567890",
            preferred_vendor_id="80000008-1234567890",
            purchase_cost="15.75",
            purchase_description="Bulk purchase of steel bolts for inventory",
            purchase_tax_code_id="80000006-1234567890",
            reorder_point=50,
            sales_description="High-quality steel bolts suitable for construction",
            sales_price="19.99",
            sales_tax_code_id="80000004-1234567890",
            sku="MPN-123456",
            unit_of_measure_set_id="80000003-1234567890",
            update_existing_transactions_cogs_account=False,
            update_existing_transactions_income_account=False,
        )
        assert_matches_type(InventoryItem, inventory_item, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncConductor) -> None:
        response = await async_client.qbd.inventory_items.with_raw_response.update(
            id="80000001-1234567890",
            revision_number="1721172183",
            conductor_end_user_id="end_usr_1234567abcdefg",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        inventory_item = await response.parse()
        assert_matches_type(InventoryItem, inventory_item, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncConductor) -> None:
        async with async_client.qbd.inventory_items.with_streaming_response.update(
            id="80000001-1234567890",
            revision_number="1721172183",
            conductor_end_user_id="end_usr_1234567abcdefg",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            inventory_item = await response.parse()
            assert_matches_type(InventoryItem, inventory_item, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncConductor) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.qbd.inventory_items.with_raw_response.update(
                id="",
                revision_number="1721172183",
                conductor_end_user_id="end_usr_1234567abcdefg",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncConductor) -> None:
        inventory_item = await async_client.qbd.inventory_items.list(
            conductor_end_user_id="end_usr_1234567abcdefg",
        )
        assert_matches_type(AsyncCursorPage[InventoryItem], inventory_item, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncConductor) -> None:
        inventory_item = await async_client.qbd.inventory_items.list(
            conductor_end_user_id="end_usr_1234567abcdefg",
            class_ids=["80000001-1234567890"],
            cursor="12345678-abcd-abcd-example-1234567890ab",
            full_names=["Kitchen:Cabinet"],
            ids=["80000001-1234567890"],
            limit=150,
            name_contains="ABC",
            name_ends_with="ABC",
            name_from="A",
            name_starts_with="ABC",
            name_to="Z",
            status="active",
            updated_after="updatedAfter",
            updated_before="updatedBefore",
        )
        assert_matches_type(AsyncCursorPage[InventoryItem], inventory_item, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncConductor) -> None:
        response = await async_client.qbd.inventory_items.with_raw_response.list(
            conductor_end_user_id="end_usr_1234567abcdefg",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        inventory_item = await response.parse()
        assert_matches_type(AsyncCursorPage[InventoryItem], inventory_item, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncConductor) -> None:
        async with async_client.qbd.inventory_items.with_streaming_response.list(
            conductor_end_user_id="end_usr_1234567abcdefg",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            inventory_item = await response.parse()
            assert_matches_type(AsyncCursorPage[InventoryItem], inventory_item, path=["response"])

        assert cast(Any, response.is_closed) is True
